unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

//
//  3.1 - 

implementation
{$R *.dfm}
uses OpenGL;
//      
Const
  AngleDl=5;
//  
var
  hrc: HGLRC;
  dc: HDC;
  //       
  AngleX, AngleY, AngleZ: integer;

//  3.1 -  
//



//
//  9.1 - 
type
  TGLTexture = record
    Height: Integer;
    Width: Integer;
    Data: array of Byte;
  end;

procedure LoadTextureFromBMP(const AFileName: String; var ATexture: TGLTexture);
var
  i, j: Integer;
  bmp: TBitmap;
begin
  bmp := TBitmap.Create;
  try
    bmp.LoadFromFile(AFileName); //    

    ATexture.Height := bmp.Height;
    ATexture.Width  := bmp.Width;

    SetLength(ATexture.Data, bmp.Width* bmp.Height*3);
    //   
    For i:=0 to bmp.Height-1 do
      For j:=0 to bmp.Width-1 do begin
        ATexture.Data[(i*bmp.Width+j)*3+0]:=GetRValue(bmp.Canvas.Pixels[j, i]);
        ATexture.Data[(i*bmp.Width+j)*3+1]:=GetGValue(bmp.Canvas.Pixels[j, i]);
        ATexture.Data[(i*bmp.Width+j)*3+2]:=GetBValue(bmp.Canvas.Pixels[j, i]);
      end;
  finally
    bmp.Free;
  end;
end;
//  9.1 - 
//

//
//  3.1 - 
procedure TForm1.FormCreate(Sender: TObject);
var
  PFD: TPixelFormatDescriptor;
  i: integer;
  Tex1: TGLTexture;
begin
  //        GetDC
  dc:=GetDC(handle);
  //   pfd
  FillChar(PFD, SizeOf(PFD),0);
  pfd.dwFlags  := PFD_DRAW_TO_WINDOW
    or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER;
  //        
  i:=ChoosePixelFormat(dc, @PFD);
  if i=0 then
    raise Exception.Create('    !');
  //      ,   
  // -  ChoosePixelFormat
  if not SetPixelFormat(dc, i, @PFD) then
    raise Exception.Create('     !');
  //      OpenGL
  //         
  hrc:=wglCreateContext(dc);
  if hrc=0 then
    raise Exception.Create('    !');
  //    
  if not wglMakeCurrent(dc, hrc) then
    raise
      Exception.Create('     !');
  glEnable(GL_DEPTH_TEST);

  //
  //  9.2 - 
  LoadTextureFromBMP('OGL64x64.bmp',  Tex1);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, Tex1.Width, Tex1.Height,
              0, GL_RGB, GL_UNSIGNED_BYTE, @Tex1.Data[0]);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
  glEnable(GL_TEXTURE_2D);
  //  9.2 - 
  //

  AngleX := -90;
end;
//  3.1 - 
//

//
//  3.5 - 
procedure TForm1.FormDestroy(Sender: TObject);
begin
  if not wglMakeCurrent(0, 0) then
    raise Exception.Create(
      '     !');
  if not wglDeleteContext(hrc) then
    raise Exception.Create('    !');
  if ReleaseDC(Form1.Handle, dc)=0 then
    raise Exception.Create('   ');
end;
//  3.5 - 
//

//
//  3.4 - 
procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_INSERT: inc(AngleX, AngleDl);
    VK_DELETE: dec(AngleX, AngleDl);
    VK_HOME: inc(AngleY, AngleDl);
    VK_END: dec(AngleY, AngleDl);
    VK_PRIOR: inc(AngleZ, AngleDl);
    VK_NEXT: dec(AngleZ, AngleDl);
  end;
  //     
  InvalidateRect(Handle, nil, False);
end;
//  3.4 - 
//


//
//  9.5 - 
procedure sphere(r: GLFloat; nx, ny: integer);
var i, ix, iy: integer;
    x, y, z: GLFloat;
begin
  for iy:=0 to ny-1 do begin
      glBegin(GL_QUAD_STRIP);
      for ix:=0 to nx do begin
          x:= r*sin (iy*pi/ny)*cos(2*ix*pi/nx);
          y:=r * sin (iy*pi/ny)*sin(2*ix*pi/nx);
          z:= r* cos(iy*pi/ny);
          glNormal3f(x, y, z);//   
          glTexCoord2f(ix/nx, iy/ny);
          glVertex3f(x, y, z);

          x:= r*sin ((iy+1)*pi/ny)*cos(2*ix*pi/nx);
          y:=r * sin ((iy+1)*pi/ny)*sin(2*ix*pi/nx);
          z:= r* cos((iy+1)*pi/ny);
          glNormal3f(x, y, z);
          glTexCoord2f(ix/nx, (iy+1)/ny);
          glVertex3f(x, y, z);
      end;
      glEnd;
    end;
end;
//  9.5 - 
//


procedure TForm1.FormPaint(Sender: TObject);
begin
  glClearColor (1, 1, 1, 1.0); //  
  glClear (GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  //      
  glMatrixMode(GL_MODELVIEW);
  //  
  glLoadIdentity;
  glRotatef(AngleX, 1.0, 0.0, 0.0);
  glRotatef(AngleY, 0.0, 1.0, 0.0);
  glRotatef(AngleZ, 0.0, 0.0, 1.0);

  //  
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  sphere(0.5, 20, 20);
  glDisable(GL_LIGHTING);
  //   

  glFlush;
  glFinish;
  SwapBuffers(DC);
end;

//
//  3.2 - 
procedure TForm1.FormResize(Sender: TObject);
begin
 //   
 glViewport(0, 0, ClientWidth, ClientHeight);

 glMatrixMode(GL_PROJECTION);
 glLoadIdentity;
 glOrtho(-1, 1, -1, 1, -1, 1);
 glMatrixMode(GL_MODELVIEW);

 //  
 InvalidateRect(Handle, nil, False);

end;
//  3.2 - 
//

end.
